__all__ = [
    "gtr.services.funds.Funds",
    "gtr.services.organisations.Organisations",
    "gtr.services.persons.Persons",
    "gtr.services.projects.Projects",
    "gtr.services.publications.Publications"
]
__version__ = "0.1.1"

from gtr.services.base import _Service
from gtr.services.funds import Funds
from gtr.services.organisations import Organisations
from gtr.services.persons import Persons
from gtr.services.projects import Projects
from gtr.services.publications import Publications



import responses
import gtr


@responses.activate
def test_fund():
    """Searching by fund id works."""
    with open("tests/results.json") as results:
        body = results.read()

    responses.add(
        responses.GET,
        "http://gtr.rcuk.ac.uk/gtr/api/funds/test",
        match_querystring=True,
        status=200,
        body=body,
        content_type="application/json")

    res = gtr.Funds().fund("test")

    assert res.status_code == 200
    assert sorted(res.json().keys()) == ["a",
                                         "b",
                                         "c",
                                         "d"]


@responses.activate
def test_funds():
    """Searching for funds works."""
    with open("tests/results.json") as results:
        body = results.read()

    responses.add(
        responses.GET,
        "http://gtr.rcuk.ac.uk/gtr/api/funds?q=test&f=test",
        match_querystring=True,
        status=200,
        body=body,
        content_type="application/json")

    res = gtr.Funds().funds("test", field="test")

    assert res.status_code == 200
    assert sorted(res.json().keys()) == ["a",
                                         "b",
                                         "c",
                                         "d"]

    responses.add(
        responses.GET,
        "http://gtr.rcuk.ac.uk/gtr/api/funds?q=test&f=fu.org.n",
        match_querystring=True,
        status=200,
        body=body,
        content_type="application/json")

    res = gtr.Funds().funds("test")

    assert res.status_code == 200
    assert sorted(res.json().keys()) == ["a",
                                         "b",
                                         "c",
                                         "d"]



"""
gtr.Projects
~~~~~~~~~~~~~~~~

This module provides a Projects object to query Project data in the Gateway To Research GTR-2 API.

"""

from .base import _Service

class Projects(_Service):
    """Search GTR-2 for projects.
    """

    _FIELD_MAP = {
        'project_ref': 'pro.gr',
        'project_title': 'pro.t',
        'project_abs': 'pro.a'
    }

    def __init__(self):
        """Construct a Projects object."""
        super(Projects, self).__init__()
        self.session = self.get_session()

    def project(self, term, **kwargs):
        """Search for a project by id.

        Args:
          term (str): Term to search for.
          kwargs (dict): additional keywords passed into
            requests.session.get params keyword.
        """
        params = kwargs
        baseuri = self._BASE_URI + 'projects/' + term
        res = self.session.get(baseuri, params=params)
        self.handle_http_error(res)
        return res

    def projects(self, term, field=None, **kwargs):
        """Search for projects. Defaults to project_title. Other fields
        are:
            project_reference
            project_abstract

        Args:
          term (str): Term to search for.
          kwargs (dict): additional keywords passed into
            requests.session.get params keyword.
        """
        params = kwargs
        params['q'] = term
        if field:
            params['f'] = self._FIELD_MAP[field]
        else:
            params['f'] = 'pro.t'
        baseuri = self._BASE_URI + 'projects'
        res = self.session.get(baseuri, params=params)
        self.handle_http_error(res)
        return res




"""
gtr.Persons
~~~~~~~~~~~~~~~~

This module provides a Persons object to query Person data in the Gateway To Research GTR-2 API.

"""

from .base import _Service

class Persons(_Service):
    """Search GTR-2 for persons.
    """

    _FIELD_MAP = {
        'first_name': 'per.fn',
        'last_name': 'per.sn',
        'first_last_name': 'per.fnsn',
        'other_name': 'per.org.n',
        'org_name': 'org.n',
        'proj_title': 'per.pro.t',
        'proj_abst': 'per.pro.abs'
    }

    def __init__(self):
        """Construct a Persons object."""
        super(Persons, self).__init__()
        self.session = self.get_session()

    def persons(self, term, field=None, **kwargs):
        """Search for persons by field. Defaults to first_name. Other fields
        are:
            last_name
            first_last_name
            other_name
            org_name
            proj_title
            proj_abstract

        Args:
          term (str): Term to search for.
          kwargs (dict): additional keywords passed into
            requests.session.get params keyword.
        """
        params = kwargs
        params['q'] = term
        if field:
            params['f'] = self._FIELD_MAP[field]
        else:
            params['f'] = 'per.fn'
        baseuri = self._BASE_URI + 'persons'
        res = self.session.get(baseuri, params=params)
        self.handle_http_error(res)
        return res



import responses
import gtr


@responses.activate
def test_publication():
    "Searching for publications by id works"

    with open("tests/results.json") as results:
        body = results.read()

    responses.add(
        responses.GET,
        "http://gtr.rcuk.ac.uk/gtr/api/outcomes/publications/glaciers",
        match_querystring=True,
        status=200,
        body=body,
        content_type="application/json")

    res = gtr.Publications().publication("glaciers")

    assert res.status_code == 200
    assert sorted(res.json().keys()) == ["a",
                                         "b",
                                         "c",
                                         "d"]


@responses.activate
def test_publications():
    """Searching for publications works."""
    with open("tests/results.json") as results:
        body = results.read()

    responses.add(
        responses.GET,
        "http://gtr.rcuk.ac.uk/gtr/api/outcomes/publications?q=test&f=title",
        match_querystring=True,
        status=200,
        body=body,
        content_type="application/json")

    res = gtr.Publications().publications("test", field="title")

    assert res.status_code == 200
    assert sorted(res.json().keys()) == ["a",
                                         "b",
                                         "c",
                                         "d"]



import pytest
import requests
import responses

import gtr


def test_Service_session():
    """Get a session using an api"""
    session = gtr._Service().get_session()
    d = session.params
    assert not d    # Empty dicts eval to false


def test_product_token():
    """Get the product version"""
    assert gtr._Service().product_token == 'gtr/{0}'.format(gtr.__version__)


def test_user_agent():
    """Check User-agent correctly assigned"""
    session = gtr._Service().get_session()
    assert session.headers['User-Agent'].startswith('gtr')
    assert 'python-requests' in session.headers['User-Agent']


@responses.activate
def test_custom_messages():
    """Check status code error messaging"""
    fakeurl = 'https://example.com'
    responses.add(responses.GET, fakeurl, status=401)
    _service = gtr._Service()
    response = _service.get_session().get(fakeurl)

    assert _service.handle_http_error(response) is None

    with pytest.raises(requests.exceptions.HTTPError) as exc:
        assert _service.handle_http_error(response,
                                          custom_messages={401: "error"})
        assert exc.value.message == 'error'

    with pytest.raises(requests.exceptions.HTTPError) as exc:
        assert _service.handle_http_error(response, raise_for_status=True)
        assert "401" in exc.value.message



"""
gtr.Funds
~~~~~~~~~~~~~~~~

This module provides a Funds object to query Fund data in the Gateway To Research GTR-2 API.

"""

from .base import _Service


class Funds(_Service):
    """Search GTR-2 for a fund or funds."""

    _field_map = {
        'title': 'fu.pro.t',
        'amount': 'fu.am',
        'org_name': 'fu.org.n',
        'type': 'fu.ty'
    }

    def __init__(self):
        """Construct a Funds object."""
        super(Funds, self).__init__()
        self.session = self.get_session()

    def fund(self, term, **kwargs):
        """Search for funds by id.

        Args:
          term (str): Fund id to search on
          kwargs (dict): additional keywords passed into
            requests.session.get params keyword.
        """
        params = kwargs
        baseuri = self._BASE_URI + 'funds/' + term
        res = self.session.get(baseuri, params=params)
        self.handle_http_error(res)
        return res

    def funds(self, term, field=None, **kwargs):
        """Search for funds matching a search term.

        Args:
          term (str): Fund id to search on
          field (str): The field to search on.
            Options are title, amount, org_name and type.
          kwargs (dict): additional keywords passed into
            requests.session.get params keyword.
        """
        params = kwargs
        params['q'] = term
        if field:
            params['f'] = field
        else:
            params['f'] = 'fu.org.n'
        baseuri = self._BASE_URI + 'funds'
        res = self.session.get(baseuri, params=params)
        self.handle_http_error(res)
        return res



import requests

from .. import __version__


class _Service(object):

    def __init__(self):
        self._BASE_URI = "http://gtr.rcuk.ac.uk/gtr/api/"

    def get_session(self, token=None, env=None):
        session = requests.Session()

        session.headers.update(
            {'User-Agent': ' '.join(
                [self.product_token, requests.utils.default_user_agent()]),
             'Accept': 'application/vnd.rcuk.gtr.json-v3'})     # Returns json only
        return session

    @property
    def product_token(self):
        """A product token for use in User-Agent headers."""
        return 'gtr/{0}'.format(__version__)

    def handle_http_error(self, response, custom_messages=None,
                          raise_for_status=False):
        if not custom_messages:
            custom_messages = {}
        if response.status_code in custom_messages.keys():
            raise requests.exceptions.HTTPError(
                custom_messages[response.status_code])
        if raise_for_status:
            response.raise_for_status()



from setuptools import setup, find_packages


def readme():
    with open('README.rst') as f:
        return f.read()

setup(name='gtr',
      version='0.1.1',
      description='A Python interface to the Gateway To Research GTR-2 API.',
      long_description=readme(),
      url='http://github.com/nestauk/gtr',
      author='James Gardiner',
      author_email='james.gardiner@nesta.org.uk',
      license='APACHEV2.0',
      packages=find_packages(),
      zip_safe=False,
      install_requires=[
          'requests==2.9.1',
      ],
      download_url='https://github.com/nesta/gtr/tarball/0.1',
      classifiers=[
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
      ])



"""
gtr.Publications
~~~~~~~~~~~~~~~~

This module provides a Publications object to query publication data in the
Gateway To Research GTR-2 API.

"""

from .base import _Service


class Publications(_Service):
    """Search GTR-2 for a publication or publications."""

    # TODO - Implement user searchable fields using field codes
    _field_map = {
        'project_id': 'pub.p',
        'abstract': 'pub.pr.a',
        'title': 'pub.pr.t',
        'isbn': 'pub.isbn',
        'issn': 'pub.issn',
        'journal_title': 'pub.jt',
        'pubmed_title': 'pub.pmid',
        'doi': 'pub.doi',
        'date_published': 'pub.date'
    }

    def __init__(self):
        """Construct a Publications object."""
        super(Publications, self).__init__()
        self.session = self.get_session()

    def publication(self, term, **kwargs):
        """Search for publication by id.

        Args:
          term (str): Publication id to search on
          kwargs (dict): additional keywords passed into
            requests.session.get params keyword.
        """
        params = kwargs
        baseuri = self._BASE_URI + 'outcomes/publications/' + term
        res = self.session.get(baseuri, params=params)
        self.handle_http_error(res)
        return res

    def publications(self, term, field=None, **kwargs):
        """Search for publications matching a search term.

        Args:
          term (str): Publication id to search on
          field (str): The field to search on.
            Options are:
                project_id
                abstract
                title
                isbn
                issn
                journal_title
                pubmed_title
                doi
                date_published


          kwargs (dict): additional keywords passed into
            requests.session.get params keyword.
        """
        params = kwargs
        params['q'] = term
        if field:
            params['f'] = field
        baseuri = self._BASE_URI + 'outcomes/publications'
        res = self.session.get(baseuri, params=params)
        self.handle_http_error(res)
        return res
